@@ -2,12 +2,13 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 1, 4)
+__version__ = (0, 1, 6)
 __all__ = [
     "Return", "Yield", "YieldFrom", "iterable", "async_iterable", "foreach", "async_foreach", 
     "through", "async_through", "flatten", "async_flatten", "map", "filter", "reduce", "zip", 
     "chunked", "iter_unique", "async_iter_unique", "wrap_iter", "wrap_aiter", "acc_step", 
-    "cut_iter", "run_gen_step", "run_gen_step_iter", "bfs_gen", "with_iter_next", "backgroud_loop", 
+    "cut_iter", "run_gen_step", "run_gen_step_iter", "as_gen_step", "bfs_gen", "with_iter_next", 
+    "backgroud_loop", 
 ]
 
 from abc import ABC, abstractmethod
@@ -15,10 +16,10 @@ from asyncio import create_task, sleep as async_sleep, to_thread
 from builtins import map as _map, filter as _filter, zip as _zip
 from collections import deque
 from collections.abc import (
-    AsyncIterable, AsyncIterator, Awaitable, Buffer, Callable, Generator, 
+    AsyncIterable, AsyncIterator, Awaitable, Buffer, Callable, Coroutine, Generator, 
     Iterable, Iterator, MutableSet, Sequence, 
 )
-from contextlib import asynccontextmanager, contextmanager
+from contextlib import asynccontextmanager, contextmanager, ExitStack, AsyncExitStack
 from dataclasses import dataclass
 from itertools import batched, pairwise
 from inspect import isawaitable, iscoroutinefunction
@@ -29,6 +30,7 @@ from typing import overload, Any, AsyncContextManager, ContextManager, Literal
 from asynctools import (
     async_filter, async_map, async_reduce, async_zip, async_batched, ensure_async, ensure_aiter, 
 )
+from decotools import optional
 from texttools import format_time
 from undefined import undefined, Undefined
 
@@ -681,6 +683,21 @@ def run_gen_step_iter(
     return process()
 
 
+@optional
+def as_gen_step(
+    func: Callable, 
+    /, 
+    async_: bool = False, 
+    iter: bool = False, 
+) -> Callable:
+    def wrapper(*args, **kwds):
+        if iter:
+            return run_gen_step_iter(func(*args, **kwds), async_=async_) # type: ignore
+        else:
+            return run_gen_step(func(*args, **kwds), async_=async_)
+    return wrapper
+
+
 @overload
 def bfs_gen[T](
     initial: T, 
@@ -769,6 +786,48 @@ def with_iter_next[T](
             pass
 
 
+@overload
+def context[T](
+    func: Callable[..., T], 
+    *ctxs: ContextManager, 
+    async_: Literal[False] = False, 
+) -> T:
+    ...
+@overload
+def context[T](
+    func: Callable[..., T] | Callable[..., Awaitable[T]], 
+    *ctxs: ContextManager | AsyncContextManager, 
+    async_: Literal[True], 
+) -> Coroutine[Any, Any, T]:
+    ...
+def context[T](
+    func: Callable[..., T] | Callable[..., Awaitable[T]], 
+    *ctxs: ContextManager | AsyncContextManager, 
+    async_: Literal[False, True] = False, 
+) -> T | Coroutine[Any, Any, T]:
+    if async_:
+        async def call():
+            args: list = []
+            add_arg = args.append
+            with ExitStack() as stack:
+                async with AsyncExitStack() as async_stack:
+                    enter = stack.enter_context
+                    async_enter = async_stack.enter_async_context
+                    for ctx in ctxs:
+                        if isinstance(ctx, AsyncContextManager):
+                            add_arg(await async_enter(ctx))
+                        else:
+                            add_arg(enter(ctx))
+                    ret = func(*args)
+                    if isawaitable(ret):
+                        ret = await ret
+                    return ret
+        return call()
+    else:
+        with ExitStack() as stack:
+            return func(*map(stack.enter_context, ctxs)) # type: ignore
+
+
 @overload
 def backgroud_loop(
     call: None | Callable = None, 
